/**
 * This file contains the initial script for the hidden window which is connected to
 * the Task Controller servlet. This is a collection of functions used to implement
 * the methods in the TaskControllerQueue class, as well as a global error handler
 * and a heartbeat mechanism to reload the frame should the request be aborted.
 *
 * Please note that several of the function and variable names and values must
 * correspond with names and values in TaskControllerQueue and TaskControllerServlet.
 */

// This is how often we verify that we've received some packet from the server.
// The interval should be larger than the server's heartbeat interval, plus
// some to allow for network delays.  The server's heartbeat interval is
// TaskControllerServlet.DEFAULT_HEARTBEAT_WAIT.
var connectionCheckInterval = 45 * 1000;      // 45 seconds

// Indicates whether the client browser is still connected to the
// TaskControllerServlet.  Do not change, remove or rename this variable
// without making the corresponding change to
// TaskControllerServlet.HEARTBEAT_SCRIPT, which sets this variable from the
// servlet.
var connectedToServer = true;

// Simple debugging support.
var debug = false;

// are we using the applet version of the task controller
var usingApplet = false;

// count of bad connection checks
var checkConnectionCount = 0;

/**
 * Perform some initialization.  Some of this was previously done outside
 * any function in this file, but some browsers (IE) weren't loading the file
 * until long after the page was delivered, and therefore not starting the
 * timer for a while.  There's now an explicit call to the initialize()
 * method after including TCStart.js in order to force the browser to read the
 * file immediately.
 * */
function initialize() {
   // Install the onload handler.
   window.onload=loaded;

   // Install the window's error handler.
   window.onerror=handleError;

   // Start checking to make sure data is still being received from the pending
   // HTTP request.  We use setTimeout() instead of setInterval() because
   // setInterval() doesn't seem to react well to clock changes.  We therefore
   // schedule each timer pop one at a time.
   window.setTimeout( "checkConnection()", connectionCheckInterval );
}

function initialize2() {

   // Install the window's error handler.
   window.onerror=handleError;

   window.onunload = handleUnload;

   usingApplet = true;

   // Start checking to make sure data is still being received from the pending
   // HTTP request.  We use setTimeout() instead of setInterval() because
   // setInterval() doesn't seem to react well to clock changes.  We therefore
   // schedule each timer pop one at a time.
   window.setTimeout( "checkConnection()", connectionCheckInterval );
}
function handleUnload() {
   try {
      if ("appletReady" in window) {
         tcapplet = window.document.applets[0];
         if (tcapplet != null) {
            tcapplet.setShowLogoff(false);
         }
      }
   }
   catch (ex) {
      window.alert("exception in unload: "+ex);
   }
}

function alertIfDebug( msg )
{
  if (debug)
  {
    alert( msg );
  }
}

/**
 * Window scope error handler.  We don't want to show javascript errors
 * to the user, but we don't want to completely ignore them either.  What we'll
 * do is reload the page, passing the error information in the query part of the
 * url.  Not much we can really do to fix things, but the servlet can at least
 * log the problem and we can be sure that the javascript error can't mess up
 * future scripts sent to this frame.
 *
 * @param text
 * @param url
 * @param line
 */
function handleError( text, url, line )
{
  alertIfDebug( "errorText="+text+"\nerrorURL="+url+"\nerrorLine="+line );

  window.location.replace(window.location.pathname+"?reason=scriptError&errorText="+
                          encodeURIComponent(text)+"&errorURL="+encodeURIComponent(url)+
                          "&errorLine="+encodeURIComponent(line));
  return true;
}

/**
 * Reload the frame in which this javascript is running.
 *
 * Do not change, remove or rename this function without also changing
 * TaskControllerServlet.RELOAD_SCRIPT, which invokes this function from the servlet.
 */
function reloadHiddenFrame()
{
  alertIfDebug( "Reloading by request of server" );
  window.location.replace(window.location.pathname+"?reason=serverInitiated");
}

/**
 * The following code handles restarting the task controller connection if
 * the pending request should somehow abort.
 *
 * The idea is that every piece of script sent from the server will set
 * connectedToServer to true.  The checkConnection function is set to be
 * called every connectionCheckInterval msecs.  The connectedToServer variable
 * is only set to true by scripts arriving from the server, so if the checkConnection
 * function finds this true, the server must still be there.  The function then
 * sets connectedToServer to be false.  The ratio of the
 * connectionCheckInterval to the heartbeat interval at the server should be set so
 * that new packets arrive more frequently than they are checked.  Therefore, if
 * the checkConnection function ever finds that connectedToServer is still false,
 * the server must not still be connected, so
 * the task controller hidden frame is reloaded.
 */
function checkConnection()
{
  alertIfDebug( "checkConnection, connectedToServer = " + connectedToServer );

  window.setTimeout( "checkConnection()", connectionCheckInterval );

  if ( connectedToServer )
  {
    connectedToServer = false;
    checkConnectionCount = 0;
  }
  else
  {
    doit = true;
    if (usingApplet) {
       // Time changes can cause this timer to pop incorrectly, which causes the
       // frame to get reloaded. To get around this we will need to get 2 failed
       // checks in a row to consider the connection lost.
       checkConnectionCount++;
       if (checkConnectionCount < 2) {
          doit = false;
       }
    }
    if (doit) {
       alertIfDebug( "Reloading since haven't heard from server in a while" );
       window.location.replace(window.location.pathname+"?reason=lostServer");
    }
  }
}

/**
 * Open a window.
 *
 * @param url     the url for the new window's content
 * @param name    the name of the new window
 * @param options the JavaScript options string describing the window
 * @param relativeWidth the percentage of the screen used for the new window's width
 * @param relativeHeight the percentage of the screen used for the new window's height
 */
function openWindow( url, name, options, relativeWidth, relativeHeight )
{
  if ( relativeWidth > 0 )
  {
    newWidth = window.screen.width * (relativeWidth / 100);
    options += ",width=" + Math.round( newWidth );
  }

  if ( relativeHeight > 0 )
  {
    newHeight = window.screen.height * (relativeHeight / 100);
    options += ",height=" + Math.round( newHeight );
  }
  return window.open( url, name, options )
}

/**
 * Refresh a window.  This means that if the window exists, its contents
 * should be reloaded.  If the window does not exist, it should be
 * created just like in the openWindow method above.
 *
 * @param name    the name of the new window
 * @param url     the url for the new window's content
 * @param options the JavaScript options string describing the window
 * @param relativeWidth the percentage of the screen used for the new window's width
 * @param relativeHeight the percentage of the screen used for the new window's height
 * @param getFocus whether the refreshed window should receive focus
 */
function refreshWindow( name, url, options, relativeWidth, relativeHeight, getFocus )
{
  if ( getFocus == "true" ) {
     theWindow = openWindow( "", name, options, relativeWidth, relativeHeight );
     // If the hmcTaskId property is present, we didn't just open the window,
     // so it's safe just to give it focus.  Otherwise we must reload it.
     if ("hmcTaskId" in theWindow) {
        theWindow.focus();
        return;
     }
  }

  theWindow = openWindow( url, name, options, relativeWidth, relativeHeight );
  if ( getFocus == "true" )
  {
    theWindow.focus();
  }
}

/**
 * This frame should never completely load.  If it does, we'll immediately
 * reload it.
 */
function loaded()
{
  alertIfDebug( "Reloading since page onload event handler driven" );
  window.location.replace(window.location.pathname+"?reason=connectionClosed");
}
